#!/bin/sh
#
# File: xas.sh
# Date: 19-Feb-2014
# By  : Kevin Esteb
#
# This file defines the environment varaibles for the XAS environment.
#
#export XAS_HOSTNAME="localhost"
#export XAS_DOMAIN="example.com"
#
#export XAS_MXPORT="25"
#export XAS_MXSERVER="localhost"
#export XAS_MXMAILER='sendmail'
#
#export XAS_MQPORT="61613"
#export XAS_MQSERVER="localhost"
#export XAS_MQLEVEL="1.0"
#
#export XAS_MSGS=".*\.msg$"
#export XAS_LOG_TYPE="console"
#export XAS_LOG_FACILITY="local6"
#
#export XAS_ROOT="/"
#export XAS_SBIN="/usr/sbin"
#export XAS_BIN="/usr/bin"
#export XAS_ETC="/etc/xas"
#export XAS_VAR="/var'
#export XAS_LIB="/var/lib/xas"
#export XAS_LOG="/var/log/xas"
#export XAS_RUN="/var/run/xas"
#export XAS_SPOOL="/var/spool/xas"
#
