@echo off
rem
rem File: xas.bat
rem Date: 10-Apr-2014
rem By  : Kevin Esteb
rem
rem This file defines the environment varaibles for the XAS environment.
rem

rem set XAS_HOSTNAME="localhost"
rem set XAS_DOMAIN="example.com"

rem set XAS_MXPORT="25"
rem set XAS_MXSERVER="localhost"
rem set XAS_MXMAILER='smtp'

rem set XAS_MQPORT="61613"
rem set XAS_MQSERVER="localhost"
rem set XAS_MQLEVEL="1.0"

rem set XAS_MSGS=".*\.msg$"
rem set XAS_LOG_TYPE="console"
rem set XAS_LOG_FACILITY="local6"

rem set XAS_ROOT="D:\XAS"
rem set XAS_SBIN="D:\XAS\sbin"
rem set XAS_BIN="D:\XAS\bin"
rem set XAS_ETC="D:\XAS\etc"
rem set XAS_VAR="D:\var"
rem set XAS_LIB="D:\var\lib"
rem set XAS_LOG="D:\var\log"
rem set XAS_RUN="D:\var\run"
rem set XAS_SPOOL="D:\var\spool"
